@@ -63,7 +63,7 @@ gst_omx_h263_enc_class_init (GstOMXH263EncClass * klass)
 
   gst_element_class_set_static_metadata (element_class,
       "OpenMAX H.263 Video Encoder",
-      "Codec/Encoder/Video",
+      "Codec/Encoder/Video/Hardware",
       "Encode H.263 video streams",
       "Sebastian Dröge <sebastian.droege@collabora.co.uk>");
 
